@@ -1,8 +1,8 @@
 /***************************************************************************
-                          LatexCode.cpp  -  description
+                    latexgenerator.cpp  -  description
                              -------------------
     begin                : Mit Jul 24 2002
-    copyright            : (C) 2002 by Andre Simon
+    copyright            : (C) 2002-2021 by Andre Simon
     email                : a.simon@mailbox.org
  ***************************************************************************/
 
@@ -42,7 +42,7 @@ LatexGenerator::LatexGenerator()
     longLineTag = "\\hspace*{\\fill}" + newLineTag;
     spacer = initialSpacer = "\\ ";
     maskWs=true;
-    maskWsBegin = "\\hlstd{";
+    maskWsBegin = "\\hl"+STY_NAME_STD+"{";
     maskWsEnd = "}";
     excludeWs=true;
     styleCommentOpen="%";
@@ -184,6 +184,9 @@ void LatexGenerator::initOutputTags()
     openTags.push_back ( "\\hl"+STY_NAME_SYM+"{" );
     openTags.push_back ( "\\hl"+STY_NAME_IPL+"{" );
 
+    openTags.push_back ( "\\hl"+STY_NAME_ERR+"{" );
+    openTags.push_back ( "\\hl"+STY_NAME_ERM+"{" );
+
     for (unsigned int i=0; i<NUMBER_BUILTIN_STATES; i++ ) {
         closeTags.push_back ( "}" );
     }
@@ -193,43 +196,57 @@ string LatexGenerator::getAttributes ( const string & elemName,
                                        const ElementStyle &elem )
 {
     ostringstream s;
+
+    string customStyle(elem.getCustomStyle());
+
     s  << "\\newcommand{\\hl"
-       << elemName
-       << "}[1]{\\textcolor[rgb]{"
-       << elem.getColour().getRed ( LATEX ) << ","
-       << elem.getColour().getGreen ( LATEX ) << ","
-       << elem.getColour().getBlue ( LATEX )
-       << "}{";
+    << elemName
+    << "}[1]{";
 
-    if ( elem.isBold() )
-        s << "\\bf{";
-    if ( elem.isItalic() )
-        s << "\\it{";
+    if (customStyle.empty()) {
 
-    s  <<"#1";
+        s   <<"\\textcolor[rgb]{"
+            << elem.getColour().getRed ( LATEX ) << ","
+            << elem.getColour().getGreen ( LATEX ) << ","
+            << elem.getColour().getBlue ( LATEX )
+            << "}{";
 
-    if ( elem.isBold() )
-        s << "}";
-    if ( elem.isItalic() )
-        s << "}";
+        if ( elem.isBold() )
+            s << "\\bf{";
+        if ( elem.isItalic() )
+            s << "\\it{";
 
-    s  <<"}}\n";
+        s  <<"#1";
+
+        if ( elem.isBold() )
+            s << "}";
+        if ( elem.isItalic() )
+            s << "}";
+
+        s  <<"}";
+    } else {
+        s << customStyle;
+    }
+    s  <<"}\n";
     return s.str();
 }
 
 
 string LatexGenerator::getNewLine()
 {
-    string nl;
+    ostringstream ss;
+
+    printSyntaxError(ss);
 
     // set wrapping arrow if previous line was wrapped
     if ( preFormatter.isWrappedLine ( lineNumber-1 ) ) {
-        nl = "\\Righttorque";
+        ss << "\\Righttorque";
     }
-    nl += ( showLineNumbers ) ? newLineTag:longLineTag;
-    return nl;
+    ss << (( showLineNumbers ) ? newLineTag:longLineTag);
+    return ss.str();
 }
 
+
 string LatexGenerator::maskCharacter ( unsigned char c )
 {
     switch ( c ) {
@@ -323,6 +340,9 @@ string LatexGenerator::getStyleDefinition()
         os << getAttributes ( STY_NAME_IPL, docStyle.getInterpolationStyle() );
         os << getAttributes ( STY_NAME_LIN, docStyle.getLineStyle() );
 
+        os << getAttributes ( STY_NAME_ERR, docStyle.getErrorStyle() );
+        os << getAttributes ( STY_NAME_ERM, docStyle.getErrorMessageStyle() );
+
         KeywordStyles styles = docStyle.getKeywordStyles();
         for ( KSIterator it=styles.begin(); it!=styles.end(); it++ ) {
             os << getAttributes ( it->first, it->second );
