@@ -127,22 +127,69 @@ Check installation.</source>
 Revise su instalación.</translation>
     </message>
     <message>
-        <location filename="mainwindow.cpp" line="+105"/>
+        <location filename="mainwindow.cpp" line="+140"/>
+        <location line="+15"/>
+        <location line="+16"/>
         <source>Initialization error</source>
         <translation type="unfinished">初始化错误</translation>
     </message>
+    <message>
+        <location line="-30"/>
+        <source>Could not read a colour theme: </source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+4"/>
+        <source>light</source>
+        <translation type="unfinished"></translation>
+    </message>
     <message>
         <location line="+1"/>
+        <source>dark</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+1"/>
+        <source>B16 light</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+1"/>
+        <source>B16 dark</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+3"/>
+        <source>Please select a Server</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+5"/>
+        <source>Could not find LSP profiles. Check installation.</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+16"/>
         <source>Could not find syntax definitions. Check installation.</source>
         <translation type="unfinished">找不到语法定义文件.请检查安装.</translation>
     </message>
     <message>
-        <location line="+77"/>
+        <location line="+124"/>
+        <source>NTFS Short Names</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+0"/>
+        <source>NTFS short names may be disabled on your volumes. Highlight can not read input with multibyte file names if no short name is available. This information will no longer bother you.</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+3"/>
         <source>Always at your service</source>
         <translation type="unfinished"></translation>
     </message>
     <message>
-        <location line="+27"/>
+        <location line="+44"/>
         <source>Select one or more files to open</source>
         <translation type="unfinished">选择一个或多个文件打开</translation>
     </message>
@@ -160,7 +207,7 @@ Revise su instalación.</translation>
         <translation type="obsolete">无法读取 &quot;%1&quot; 的Tag信息</translation>
     </message>
     <message>
-        <location line="+635"/>
+        <location line="+750"/>
         <location line="+31"/>
         <source>Output error</source>
         <translation type="unfinished">输出错误</translation>
@@ -176,12 +223,12 @@ Revise su instalación.</translation>
         <translation type="unfinished">你必须等译一个style输出文件！</translation>
     </message>
     <message>
-        <location line="+32"/>
+        <location line="+52"/>
         <source>Processing %1 (%2/%3)</source>
         <translation type="unfinished"></translation>
     </message>
     <message>
-        <location line="+6"/>
+        <location line="+10"/>
         <source>Language definition error</source>
         <translation type="unfinished">语言定义错误</translation>
     </message>
@@ -214,35 +261,35 @@ Lua Syntax error: %2</source>
         <translation type="unfinished"></translation>
     </message>
     <message>
-        <location line="+78"/>
+        <location line="+175"/>
         <source>Converted %1 files in %2 ms</source>
         <translation type="unfinished">文件%1 转换完毕，用时 %2 ms</translation>
     </message>
     <message>
-        <location line="+123"/>
+        <location line="+204"/>
         <source>Conversion of &quot;%1&quot; not possible.</source>
         <translation type="unfinished">No es posible la conversión de &quot;%1&quot;.</translation>
     </message>
     <message>
         <location line="+0"/>
-        <location line="+118"/>
-        <location line="+25"/>
+        <location line="+148"/>
+        <location line="+55"/>
         <source>clipboard data</source>
         <translation type="unfinished">剪贴板数据</translation>
     </message>
     <message>
-        <location line="-76"/>
+        <location line="-124"/>
         <source>%1 options</source>
         <translation type="unfinished">%1 选项</translation>
     </message>
     <message>
-        <location line="+37"/>
-        <location line="+6"/>
+        <location line="+41"/>
+        <location line="+11"/>
         <source>(user script)</source>
         <translation type="unfinished"></translation>
     </message>
     <message>
-        <location line="+7"/>
+        <location line="+16"/>
         <source>Preview (%1):</source>
         <translation type="unfinished">预览 (%1):</translation>
     </message>
@@ -256,17 +303,47 @@ Lua Syntax error: %2</source>
         <source>Current theme: %1 %2</source>
         <translation type="unfinished"></translation>
     </message>
+    <message>
+        <location line="+3"/>
+        <source>Contrast: %1 %2</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+452"/>
+        <source>Language server initialization successful</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+5"/>
+        <source>Language server connection failed</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+2"/>
+        <source>Language server initialization failed</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+28"/>
+        <source>Select workspace directory</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+4"/>
+        <source>Choose the Language Server executable</source>
+        <translation type="unfinished"></translation>
+    </message>
     <message>
         <source>Current syntax: %1</source>
         <translation type="obsolete">当前语言: %1</translation>
     </message>
     <message>
-        <location line="+22"/>
+        <location line="-442"/>
         <source>Preview of &quot;%1&quot; not possible.</source>
         <translation type="unfinished">&quot;%1&quot;.的预览不可用.</translation>
     </message>
     <message>
-        <location line="+249"/>
+        <location line="+250"/>
         <source>Some plug-in effects may not be visible in the preview.</source>
         <translation type="unfinished"></translation>
     </message>
@@ -275,7 +352,7 @@ Lua Syntax error: %2</source>
         <translation type="obsolete">选择一个ctags文件</translation>
     </message>
     <message>
-        <location line="-214"/>
+        <location line="-206"/>
         <location line="+4"/>
         <location line="+4"/>
         <location line="+5"/>
@@ -374,7 +451,9 @@ Please send a note to as (at) andre-simon (dot) de if you have issues during tra
     <message>
         <location line="+3"/>
         <location line="+359"/>
-        <location line="+649"/>
+        <location line="+150"/>
+        <location line="+65"/>
+        <location line="+625"/>
         <location line="+269"/>
         <location line="+108"/>
         <location line="+295"/>
@@ -382,7 +461,7 @@ Please send a note to as (at) andre-simon (dot) de if you have issues during tra
         <translation>...</translation>
     </message>
     <message>
-        <location line="-1645"/>
+        <location line="-1836"/>
         <source>Save output in the input file directories.</source>
         <translation>将输出保存在输入路径中.</translation>
     </message>
@@ -396,7 +475,7 @@ Please send a note to as (at) andre-simon (dot) de if you have issues during tra
         <translation type="obsolete">Opciones de salida</translation>
     </message>
     <message>
-        <location line="+440"/>
+        <location line="+616"/>
         <source>General</source>
         <translation>通用</translation>
     </message>
@@ -447,7 +526,7 @@ Please send a note to as (at) andre-simon (dot) de if you have issues during tra
         <translation></translation>
     </message>
     <message>
-        <location line="+24"/>
+        <location line="+39"/>
         <source>Add line numbers to the output.</source>
         <translation>给输出添加代码行号.</translation>
     </message>
@@ -508,7 +587,7 @@ Removes Unicode BOM mark.</source>
         <translation type="vanished">设置输出文件的编码格式.</translation>
     </message>
     <message>
-        <location line="+1484"/>
+        <location line="+1480"/>
         <source>&amp;Plug-Ins</source>
         <translation type="unfinished"></translation>
     </message>
@@ -517,7 +596,7 @@ Removes Unicode BOM mark.</source>
         <translation type="obsolete">Codificación:</translation>
     </message>
     <message>
-        <location line="-1455"/>
+        <location line="-1451"/>
         <source>Select or define the encoding.
 The result has to match the input file encoding.</source>
         <translation>选择或者定义编码格式.
@@ -721,12 +800,11 @@ See ctags.sf.net for details.</source>
         <translation>杂项</translation>
     </message>
     <message>
-        <location line="+19"/>
         <source>Add an achor to each line.</source>
-        <translation>给每一行代码添加锚点..</translation>
+        <translation type="vanished">给每一行代码添加锚点..</translation>
     </message>
     <message>
-        <location line="+3"/>
+        <location line="+22"/>
         <source>Add line anchors</source>
         <translation>给每一行代码添加Anchor(锚点)</translation>
     </message>
@@ -821,12 +899,12 @@ See ctags.sf.net for details.</source>
         <translation>编码设置：</translation>
     </message>
     <message>
-        <location line="-740"/>
+        <location line="-931"/>
         <source>Output destination:</source>
         <translation>输出路径：</translation>
     </message>
     <message>
-        <location line="+966"/>
+        <location line="+1157"/>
         <location line="+278"/>
         <location line="+108"/>
         <location line="+295"/>
@@ -846,7 +924,7 @@ See ctags.sf.net for details.</source>
         <translation type="obsolete">读取ctags文件：</translation>
     </message>
     <message>
-        <location line="-1933"/>
+        <location line="-2124"/>
         <source>Highlight</source>
         <translation>Highlight</translation>
     </message>
@@ -916,12 +994,11 @@ See ctags.sf.net for details.</source>
         <translation>描述</translation>
     </message>
     <message>
-        <location line="-124"/>
         <source>Shows decription of a selected plug-in script.</source>
-        <translation>显示选中插件的描述.</translation>
+        <translation type="vanished">显示选中插件的描述.</translation>
     </message>
     <message>
-        <location line="+3"/>
+        <location line="-121"/>
         <source>Plug-In description</source>
         <translation>插件描述</translation>
     </message>
@@ -943,12 +1020,17 @@ See ctags.sf.net for details.</source>
         <translation>选择传递给插件的文件.</translation>
     </message>
     <message>
-        <location line="+203"/>
+        <location line="+77"/>
+        <source>Shows description of a selected script.</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+302"/>
         <source>BBCode</source>
         <translation>BBCode</translation>
     </message>
     <message>
-        <location line="+42"/>
+        <location line="+57"/>
         <source>Set line numbering start.</source>
         <translation type="unfinished"></translation>
     </message>
@@ -1131,7 +1213,7 @@ You can select the stylesheets in your word processor to reformat additional tex
         <translation>高亮主题：</translation>
     </message>
     <message>
-        <location line="+60"/>
+        <location line="+47"/>
         <source>Select a colour theme.</source>
         <translation>选择一个用于高亮的颜色主题.</translation>
     </message>
@@ -1142,7 +1224,7 @@ This feature is enabled tor C, C++, C# and Java code.</source>
 这项功能仅支持 C, C++, C# 和 Java.</translation>
     </message>
     <message>
-        <location line="+21"/>
+        <location line="+24"/>
         <source>Reformat:</source>
         <translation>格式化代码:</translation>
     </message>
@@ -1237,7 +1319,7 @@ Set the width to 0 to keep tabs.</source>
         <translation type="vanished">字体大小：</translation>
     </message>
     <message>
-        <location line="+55"/>
+        <location line="+61"/>
         <source>Enter the font size.</source>
         <translation>输入字体大小.</translation>
     </message>
@@ -1253,7 +1335,7 @@ Set the width to 0 to keep tabs.</source>
         <translation>停靠浮动面板(&amp;D)</translation>
     </message>
     <message>
-        <location line="-2440"/>
+        <location line="-2627"/>
         <source>Preview</source>
         <translation>预览</translation>
     </message>
@@ -1268,12 +1350,11 @@ Set the width to 0 to keep tabs.</source>
         <translation>转换文件</translation>
     </message>
     <message>
-        <location line="+10"/>
         <source>Copy highlighted code of the seleted file into the clipboard.</source>
-        <translation>将选中文件的高亮代码复制到剪贴板中.</translation>
+        <translation type="vanished">将选中文件的高亮代码复制到剪贴板中.</translation>
     </message>
     <message>
-        <location line="+15"/>
+        <location line="+25"/>
         <source>Clipboard</source>
         <translation>剪贴板</translation>
     </message>
@@ -1324,7 +1405,12 @@ Set the width to 0 to keep tabs.</source>
         <translation type="unfinished"></translation>
     </message>
     <message>
-        <location line="+152"/>
+        <location line="+67"/>
+        <source>Copy highlighted code of the selected file into the clipboard.</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+85"/>
         <source>Paste code from clipboard and copy the output back in one step</source>
         <translation type="unfinished"></translation>
     </message>
@@ -1344,7 +1430,12 @@ Set the width to 0 to keep tabs.</source>
         <translation type="unfinished"></translation>
     </message>
     <message>
-        <location line="+167"/>
+        <location line="+97"/>
+        <source>Shows description of a selected plug-in script.</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+70"/>
         <source>Scripts</source>
         <translation type="unfinished"></translation>
     </message>
@@ -1375,17 +1466,127 @@ Set the width to 0 to keep tabs.</source>
         <translation type="unfinished"></translation>
     </message>
     <message>
-        <location line="+18"/>
-        <source>Shows decription of a selected script.</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <location line="+3"/>
+        <location line="+21"/>
         <source>Script description</source>
         <translation type="unfinished"></translation>
     </message>
     <message>
-        <location line="+252"/>
+        <location line="+11"/>
+        <source>Configure the LSP Client</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+7"/>
+        <source>LSP</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+8"/>
+        <source>Language Server:</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+7"/>
+        <source>Select a Language Server profile.</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+17"/>
+        <source>Executable:</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+7"/>
+        <source>Path of the server executable.</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+13"/>
+        <source>Select the server executable.</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+14"/>
+        <source>Print LSP messages to standard error</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+3"/>
+        <source>Output Logs</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+7"/>
+        <source>Start an initialization request.</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+3"/>
+        <source>Check Capabilities</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+18"/>
+        <source>Workspace:</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+7"/>
+        <source>The workspace directory.</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+13"/>
+        <source>Select the workspace directory of your project.</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+12"/>
+        <source>Run hover requests to add tooltips to your output (HTML only).</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+3"/>
+        <source>Hover</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+10"/>
+        <source>Improve token recognition with semantic highlighting.</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+3"/>
+        <source>Semantic Highlighting</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+10"/>
+        <source>Show syntax errors and messages (depends on output format).</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+3"/>
+        <source>Show Syntax Errors</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+128"/>
+        <source>ANSI ESC</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+5"/>
+        <source>XTerm256 ESC</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+5"/>
+        <source>Truecolor ESC</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+129"/>
         <source>Generate output without version information comment.</source>
         <translation type="unfinished"></translation>
     </message>
@@ -1400,7 +1601,12 @@ Set the width to 0 to keep tabs.</source>
         <translation type="unfinished"></translation>
     </message>
     <message>
-        <location line="+417"/>
+        <location line="+353"/>
+        <source>Add an anchor to each line.</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+64"/>
         <source>Add HTML MIME Type when copying code to the clipboard</source>
         <translation type="unfinished"></translation>
     </message>
@@ -1435,17 +1641,7 @@ Set the width to 0 to keep tabs.</source>
         <translation type="unfinished"></translation>
     </message>
     <message>
-        <location line="+274"/>
-        <source>Toggle classic theme or Base16 theme selection.</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <location line="+3"/>
-        <source>Base16</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <location line="+42"/>
+        <location line="+309"/>
         <source>Reformat and indent your code.
 This feature is enabled for C, C++, C# and Java code.</source>
         <translation type="unfinished"></translation>
@@ -1466,7 +1662,12 @@ This feature is enabled for C, C++, C# and Java code.</source>
         <translation>选择或者输入字体名. HTML支持用逗号分隔的字体列表.</translation>
     </message>
     <message>
-        <location line="+12"/>
+        <location line="+3"/>
+        <source>DejaVu Sans Mono</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location line="+15"/>
         <source>Font si&amp;ze:</source>
         <translation type="unfinished"></translation>
     </message>
@@ -1527,6 +1728,11 @@ This feature is enabled for C, C++, C# and Java code.</source>
         <oldsource>About &amp;translations</oldsource>
         <translation type="unfinished">关于翻译(&amp;T)</translation>
     </message>
+    <message>
+        <location line="+26"/>
+        <source>&amp;File access trace (W32)</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>ShowTextFile</name>
